@@ -210,13 +210,10 @@ builder_cache_get_checksum (BuilderCache *self)
   return self->checksum;
 }
 
-static char *
-get_ref (BuilderCache *self, const char *stage)
+static void
+append_escaped_stage (GString *s,
+                      const char *stage)
 {
-  GString *s = g_string_new (self->branch);
-
-  g_string_append_c (s, '/');
-
   while (*stage)
     {
       char c = *stage++;
@@ -228,6 +225,16 @@ get_ref (BuilderCache *self, const char *stage)
       else
         g_string_append_printf (s, "%x", c);
     }
+}
+
+static char *
+get_ref (BuilderCache *self, const char *stage)
+{
+  GString *s = g_string_new (self->branch);
+
+  g_string_append_c (s, '/');
+
+  append_escaped_stage (s, stage);
 
   return g_string_free (s, FALSE);
 }
@@ -376,11 +383,13 @@ builder_cache_lookup (BuilderCache *self,
 {
   g_autofree char *commit = NULL;
   g_autofree char *ref = NULL;
+  g_autoptr(GString) s = g_string_new ("");
 
   g_free (self->stage);
   self->stage = g_strdup (stage);
 
-  g_hash_table_remove (self->unused_stages, stage);
+  append_escaped_stage (s, stage);
+  g_hash_table_remove (self->unused_stages, s->str);
 
   g_free (self->current_checksum);
   self->current_checksum = g_strdup (g_checksum_get_string (self->checksum));
@@ -1210,6 +1219,7 @@ builder_cache_disable_lookups (BuilderCache *self)
 
 gboolean
 builder_gc (BuilderCache *self,
+            gboolean      prune_unused_stages,
             GError      **error)
 {
   gint objects_total;
@@ -1218,20 +1228,23 @@ builder_gc (BuilderCache *self,
   GHashTableIter iter;
   gpointer key, value;
 
-  g_hash_table_iter_init (&iter, self->unused_stages);
-  while (g_hash_table_iter_next (&iter, &key, &value))
+  if (prune_unused_stages)
     {
-      const char *unused_stage = (const char *) key;
-      g_autofree char *unused_ref = get_ref (self, unused_stage);
+      g_hash_table_iter_init (&iter, self->unused_stages);
+      while (g_hash_table_iter_next (&iter, &key, &value))
+        {
+          const char *unused_stage = (const char *) key;
+          g_autofree char *unused_ref = get_ref (self, unused_stage);
 
-      g_debug ("Removing unused ref %s", unused_ref);
+          g_debug ("Removing unused ref %s", unused_ref);
 
-      if (!ostree_repo_set_ref_immediate (self->repo,
-                                          NULL,
-                                          unused_ref,
-                                          NULL,
-                                          NULL, error))
-        return FALSE;
+          if (!ostree_repo_set_ref_immediate (self->repo,
+                                              NULL,
+                                              unused_ref,
+                                              NULL,
+                                              NULL, error))
+            return FALSE;
+        }
     }
 
   g_print ("Pruning cache\n");
@@ -1336,6 +1349,13 @@ builder_cache_checksum_uint32 (BuilderCache *self,
   g_checksum_update (self->checksum, v, 4);
 }
 
+void
+builder_cache_checksum_random (BuilderCache *self)
+{
+  builder_cache_checksum_uint32 (self, g_random_int ());
+  builder_cache_checksum_uint32 (self, g_random_int ());
+}
+
 void
 builder_cache_checksum_uint64 (BuilderCache *self,
                                guint64       val)
