@@ -46,6 +46,7 @@ struct BuilderContext
   GFile          *base_dir; /* directory with json manifest, origin for source files */
   char           *state_subdir;
   SoupSession    *soup_session;
+  CURL           *curl_session;
   char           *arch;
   char           *stop_at;
 
@@ -77,6 +78,9 @@ struct BuilderContext
   gboolean        use_rofiles;
   gboolean        have_rofiles;
   gboolean        run_tests;
+  gboolean        no_shallow_clone;
+
+  BuilderSdkConfig *sdk_config;
 };
 
 typedef struct
@@ -112,6 +116,7 @@ builder_context_finalize (GObject *object)
   g_clear_object (&self->base_dir);
   g_clear_object (&self->soup_session);
   g_clear_object (&self->options);
+  g_clear_object (&self->sdk_config);
   g_free (self->arch);
   g_free (self->state_subdir);
   g_free (self->stop_at);
@@ -122,6 +127,10 @@ builder_context_finalize (GObject *object)
   g_clear_pointer (&self->sources_dirs, g_ptr_array_unref);
   g_clear_pointer (&self->sources_urls, g_ptr_array_unref);
 
+  curl_easy_cleanup (self->curl_session);
+  self->curl_session = NULL;
+  curl_global_cleanup ();
+
   G_OBJECT_CLASS (builder_context_parent_class)->finalize (object);
 }
 
@@ -353,6 +362,7 @@ builder_context_set_sources_urls (BuilderContext *self,
 gboolean
 builder_context_download_uri (BuilderContext *self,
                               const char     *url,
+                              const char    **mirrors,
                               GFile          *dest,
                               const char     *checksums[BUILDER_CHECKSUMS_LEN],
                               GChecksumType   checksums_type[BUILDER_CHECKSUMS_LEN],
@@ -360,6 +370,7 @@ builder_context_download_uri (BuilderContext *self,
 {
   int i;
   g_autoptr(SoupURI) original_uri = soup_uri_new (url);
+  g_autoptr(GError) first_error = NULL;
 
   if (original_uri == NULL)
     return flatpak_fail (error, _("Could not parse URI “%s”"), url);
@@ -382,11 +393,11 @@ builder_context_download_uri (BuilderContext *self,
           if (builder_download_uri (mirror_uri,
                                     dest,
                                     checksums, checksums_type,
-                                    builder_context_get_soup_session (self),
+                                    builder_context_get_curl_session (self),
                                     &my_error))
             return TRUE;
 
-          if (!g_error_matches (my_error, SOUP_HTTP_ERROR, SOUP_STATUS_NOT_FOUND))
+          if (!g_error_matches (my_error, BUILDER_CURL_ERROR, CURLE_REMOTE_FILE_NOT_FOUND))
             g_warning ("Error downloading from mirror: %s\n", my_error->message);
         }
     }
@@ -394,9 +405,41 @@ builder_context_download_uri (BuilderContext *self,
   if (!builder_download_uri (original_uri,
                              dest,
                              checksums, checksums_type,
-                             builder_context_get_soup_session (self),
-                             error))
-    return FALSE;
+                             builder_context_get_curl_session (self),
+                             &first_error))
+    {
+      gboolean mirror_ok = FALSE;
+
+      if (mirrors != NULL && mirrors[0] != NULL)
+        {
+          g_print ("Error downloading, trying mirrors\n");
+          for (i = 0; mirrors[i] != NULL; i++)
+            {
+              g_autoptr(GError) mirror_error = NULL;
+              g_autoptr(SoupURI) mirror_uri = soup_uri_new (mirrors[i]);
+              g_print ("Trying mirror %s\n", mirrors[i]);
+              if (!builder_download_uri (mirror_uri,
+                                         dest,
+                                         checksums, checksums_type,
+                                         builder_context_get_curl_session (self),
+                                         &mirror_error))
+                {
+                  g_print ("Error downloading mirror: %s\n", mirror_error->message);
+                }
+              else
+                {
+                  mirror_ok = TRUE;
+                  break;
+                }
+            }
+        }
+
+      if (!mirror_ok)
+        {
+          g_propagate_error (error, g_steal_pointer (&first_error));
+          return FALSE;
+        }
+    }
 
   return TRUE;
 }
@@ -417,7 +460,8 @@ char *
 builder_context_get_checksum_for (BuilderContext *self,
                                   const char *name)
 {
-  g_autoptr(GFile) checksum_file = g_file_get_child (self->checksums_dir, name);
+  g_autofree char *checksum_name = g_strdup_printf ("%s-%s", builder_context_get_arch (self), name);
+  g_autoptr(GFile) checksum_file = g_file_get_child (self->checksums_dir, checksum_name);
   g_autofree gchar *checksum = NULL;
 
   if (!g_file_get_contents (flatpak_file_get_path_cached (checksum_file), &checksum, NULL, NULL))
@@ -432,7 +476,8 @@ builder_context_set_checksum_for (BuilderContext  *self,
                                   const char      *checksum,
                                   GError         **error)
 {
-  g_autoptr(GFile) checksum_file = g_file_get_child (self->checksums_dir, name);
+  g_autofree char *checksum_name = g_strdup_printf ("%s-%s", builder_context_get_arch (self), name);
+  g_autoptr(GFile) checksum_file = g_file_get_child (self->checksums_dir, checksum_name);
 
   if (!flatpak_mkdir_p (self->checksums_dir,
                         NULL, error))
@@ -494,6 +539,15 @@ builder_context_get_soup_session (BuilderContext *self)
   return self->soup_session;
 }
 
+CURL *
+builder_context_get_curl_session (BuilderContext *self)
+{
+  if (self->curl_session == NULL)
+    self->curl_session = flatpak_create_curl_session ("flatpak-builder " PACKAGE_VERSION);
+
+  return self->curl_session;
+}
+
 const char *
 builder_context_get_arch (BuilderContext *self)
 {
@@ -677,7 +731,7 @@ static char *rofiles_unmount_path = NULL;
 static void
 rofiles_umount_handler (int signum)
 {
-  char *argv[] = { "fusermount", "-u", NULL,
+  char *argv[] = { "fusermount", "-uz", NULL,
                      NULL };
 
   argv[2] = rofiles_unmount_path;
@@ -861,6 +915,19 @@ builder_context_set_run_tests (BuilderContext *self,
   self->run_tests = run_tests;
 }
 
+void
+builder_context_set_no_shallow_clone (BuilderContext *self,
+                                      gboolean        no_shallow_clone)
+{
+  self->no_shallow_clone = no_shallow_clone;
+}
+
+gboolean
+builder_context_get_no_shallow_clone (BuilderContext *self)
+{
+  return self->no_shallow_clone;
+}
+
 gboolean
 builder_context_get_rebuild_on_sdk_change (BuilderContext *self)
 {
@@ -875,36 +942,64 @@ builder_context_set_rebuild_on_sdk_change (BuilderContext *self,
 }
 
 gboolean
-builder_context_enable_ccache (BuilderContext *self,
-                               GError        **error)
+builder_context_set_enable_ccache (BuilderContext *self,
+                                   gboolean        enable,
+                                   GError        **error)
 {
-  g_autofree char *ccache_path = g_file_get_path (self->ccache_dir);
-  g_autofree char *ccache_bin_path = g_build_filename (ccache_path, "bin", NULL);
   int i;
-  static const char *compilers[] = {
-    "cc",
-    "c++",
-    "gcc",
-    "g++"
-  };
 
-  if (g_mkdir_with_parents (ccache_bin_path, 0755) != 0)
-    {
-      glnx_set_error_from_errno (error);
-      return FALSE;
-    }
+  self->use_ccache = !!enable;
 
-  for (i = 0; i < G_N_ELEMENTS (compilers); i++)
+  if (enable)
     {
-      const char *symlink_path = g_build_filename (ccache_bin_path, compilers[i], NULL);
-      if (symlink ("/usr/bin/ccache", symlink_path) && errno != EEXIST)
+      g_autofree char *ccache_path = g_file_get_path (self->ccache_dir);
+      g_autofree char *ccache_bin_path = g_build_filename (ccache_path, "bin", NULL);
+      static const char *compilers[] = {
+        "cc",
+        "c++",
+        "gcc",
+        "g++"
+      };
+
+      if (g_mkdir_with_parents (ccache_bin_path, 0755) != 0)
         {
           glnx_set_error_from_errno (error);
           return FALSE;
         }
-    }
 
-  self->use_ccache = TRUE;
+      for (i = 0; i < G_N_ELEMENTS (compilers); i++)
+        {
+          const char *symlink_path = g_build_filename (ccache_bin_path, compilers[i], NULL);
+          if (symlink ("/usr/bin/ccache", symlink_path) && errno != EEXIST)
+            {
+              glnx_set_error_from_errno (error);
+              return FALSE;
+            }
+        }
+    }
+  else
+    {
+      g_autoptr(GFile) ccache_disabled_dir = g_file_get_child (self->ccache_dir, "disabled");
+      g_autofree char *ccache_disabled_path = g_file_get_path (ccache_disabled_dir);
+      g_autofree char *ccache_config_path = g_build_filename (ccache_disabled_path, "ccache.conf", NULL);
+      g_autoptr(GError) my_error = NULL;
+
+      if (g_mkdir_with_parents (ccache_disabled_path, 0755) != 0)
+        {
+          glnx_set_error_from_errno (error);
+          return FALSE;
+        }
+
+      if (!g_file_set_contents (ccache_config_path, "disable = true\n", -1, &my_error))
+        {
+          if (!g_error_matches (my_error, G_FILE_ERROR, G_FILE_ERROR_EXIST))
+            {
+              g_propagate_error (error, g_steal_pointer (&my_error));
+              return FALSE;
+            }
+          g_clear_error (&my_error);
+        }
+    }
 
   return TRUE;
 }
@@ -914,6 +1009,7 @@ builder_context_extend_env (BuilderContext *self,
                             char          **envp)
 {
   g_autofree char *path = NULL;
+  const char *ccache_dir = NULL;
 
   path = g_strdup (g_environ_getenv (envp, "PATH"));
   if (path == NULL)
@@ -924,14 +1020,47 @@ builder_context_extend_env (BuilderContext *self,
       char *new_path = g_strdup_printf ("/run/ccache/bin:%s", path);
       g_free (path);
       path = new_path;
-      envp = g_environ_setenv (envp, "CCACHE_DIR", "/run/ccache", TRUE);
+      ccache_dir = "/run/ccache";
+    }
+  else
+    {
+      ccache_dir = "/run/ccache/disabled";
     }
 
+  envp = g_environ_setenv (envp, "CCACHE_DIR", ccache_dir, TRUE);
   envp = g_environ_setenv (envp, "PATH", path, TRUE);
 
   return envp;
 }
 
+gboolean
+builder_context_load_sdk_config (BuilderContext   *self,
+                                 const char       *sdk_path,
+                                 GError          **error)
+{
+  g_autoptr(GFile) root = g_file_new_for_path (sdk_path);
+  g_autoptr(GFile) config_file = g_file_resolve_relative_path (root, "files/etc/flatpak-builder/defaults.json");
+  g_autoptr(GError) local_error = NULL;
+  g_autoptr(BuilderSdkConfig) sdk_config = NULL;
+
+  sdk_config = builder_sdk_config_from_file (config_file, &local_error);
+  if (sdk_config == NULL &&
+      !g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
+    {
+      g_propagate_error (error, g_steal_pointer (&local_error));
+      return FALSE;
+    }
+
+  g_set_object (&self->sdk_config, sdk_config);
+  return TRUE;
+}
+
+BuilderSdkConfig *
+builder_context_get_sdk_config (BuilderContext *self)
+{
+  return self->sdk_config;
+}
+
 BuilderContext *
 builder_context_new (GFile *run_dir,
                      GFile *app_dir,
