@@ -23,8 +23,10 @@
 
 #include <gio/gio.h>
 #include <libsoup/soup.h>
+#include <curl/curl.h>
 #include "builder-options.h"
 #include "builder-utils.h"
+#include "builder-sdk-config.h"
 
 G_BEGIN_DECLS
 
@@ -62,11 +64,13 @@ void            builder_context_set_sources_urls (BuilderContext *self,
                                                   GPtrArray      *sources_urls);
 gboolean        builder_context_download_uri (BuilderContext *self,
                                               const char     *url,
+                                              const char    **mirrors,
                                               GFile          *dest,
                                               const char     *checksums[BUILDER_CHECKSUMS_LEN],
                                               GChecksumType   checksums_type[BUILDER_CHECKSUMS_LEN],
                                               GError        **error);
 SoupSession *   builder_context_get_soup_session (BuilderContext *self);
+CURL *          builder_context_get_curl_session (BuilderContext *self);
 const char *    builder_context_get_arch (BuilderContext *self);
 void            builder_context_set_arch (BuilderContext *self,
                                           const char     *arch);
@@ -119,8 +123,9 @@ gboolean        builder_context_set_checksum_for (BuilderContext  *self,
 BuilderContext *builder_context_new (GFile *run_dir,
                                      GFile *app_dir,
                                      const char *state_subdir);
-gboolean        builder_context_enable_ccache (BuilderContext *self,
-                                               GError        **error);
+gboolean        builder_context_set_enable_ccache (BuilderContext *self,
+                                                   gboolean        enabled,
+                                                   GError        **error);
 gboolean        builder_context_enable_rofiles (BuilderContext *self,
                                                 GError        **error);
 gboolean        builder_context_disable_rofiles (BuilderContext *self,
@@ -132,9 +137,18 @@ void            builder_context_set_use_rofiles (BuilderContext *self,
 gboolean        builder_context_get_run_tests (BuilderContext *self);
 void            builder_context_set_run_tests (BuilderContext *self,
                                                gboolean run_tests);
+void            builder_context_set_no_shallow_clone (BuilderContext *self,
+                                                      gboolean        no_shallow_clone);
+gboolean        builder_context_get_no_shallow_clone (BuilderContext *self);
 char **         builder_context_extend_env (BuilderContext *self,
                                             char          **envp);
 
+gboolean        builder_context_load_sdk_config (BuilderContext       *self,
+                                                 const char           *sdk_path,
+                                                 GError              **error);
+
+BuilderSdkConfig * builder_context_get_sdk_config (BuilderContext *self);
+
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (BuilderContext, g_object_unref)
 
 G_END_DECLS
