@@ -41,6 +41,7 @@ static gboolean opt_disable_cache;
 static gboolean opt_disable_tests;
 static gboolean opt_disable_rofiles;
 static gboolean opt_download_only;
+static gboolean opt_no_shallow_clone;
 static gboolean opt_bundle_sources;
 static gboolean opt_build_only;
 static gboolean opt_finish_only;
@@ -110,9 +111,7 @@ static GOptionEntry entries[] = {
   { "repo", 0, 0, G_OPTION_ARG_STRING, &opt_repo, "Repo to export into", "DIR"},
   { "subject", 's', 0, G_OPTION_ARG_STRING, &opt_subject, "One line subject (passed to build-export)", "SUBJECT" },
   { "body", 'b', 0, G_OPTION_ARG_STRING, &opt_body, "Full description (passed to build-export)", "BODY" },
-#ifdef FLATPAK_ENABLE_P2P
   { "collection-id", 0, 0, G_OPTION_ARG_STRING, &opt_collection_id, "Collection ID (passed to build-export)", "COLLECTION-ID" },
-#endif  /* FLATPAK_ENABLE_P2P */
   { "gpg-sign", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_key_ids, "GPG Key ID to sign the commit with", "KEY-ID"},
   { "gpg-homedir", 0, 0, G_OPTION_ARG_STRING, &opt_gpg_homedir, "GPG Homedir to use when looking for keyrings", "HOMEDIR"},
   { "force-clean", 0, 0, G_OPTION_ARG_NONE, &opt_force_clean, "Erase previous contents of DIRECTORY", NULL },
@@ -133,6 +132,7 @@ static GOptionEntry entries[] = {
   { "installation", 0, 0, G_OPTION_ARG_STRING, &opt_installation, "Install dependencies in a specific system-wide installation", "NAME" },
   { "state-dir", 0, 0, G_OPTION_ARG_FILENAME, &opt_state_dir, "Use this directory for state instead of .flatpak-builder", "PATH" },
   { "assumeyes", 'y', 0, G_OPTION_ARG_NONE, &opt_yes, N_("Automatically answer yes for all questions"), NULL },
+  { "no-shallow-clone", 0, 0, G_OPTION_ARG_NONE, &opt_no_shallow_clone, "Don't use shallow clones when mirroring git repos", NULL },
   { NULL }
 };
 
@@ -275,13 +275,10 @@ do_install (BuilderContext *build_context,
   else
     g_ptr_array_add (args, g_strdup ("--system"));
 
-  if (opt_user)
-    g_ptr_array_add (args, g_strdup ("-y"));
+  g_ptr_array_add (args, g_strdup ("-y"));
 
   g_ptr_array_add (args, g_strdup ("--reinstall"));
 
-  g_ptr_array_add (args, g_strdup ("--subpath="));
-
   ref = flatpak_build_untyped_ref (id, branch,
                                    builder_context_get_arch (build_context));
 
@@ -302,6 +299,70 @@ do_install (BuilderContext *build_context,
   return TRUE;
 }
 
+static gboolean
+git (char   **output,
+     GError **error,
+     ...)
+{
+  gboolean res;
+  va_list ap;
+
+  if (output != NULL)
+    *output = NULL;
+
+  va_start (ap, error);
+  res = flatpak_spawn (NULL, output, 0, error, "git", ap);
+  va_end (ap);
+
+  if (output != NULL &&
+      (*output != NULL && *output[0] == '\0'))
+    {
+      g_free (*output);
+      *output = NULL;
+    }
+
+  return res;
+}
+
+static char *
+trim_linefeed (char *str)
+{
+  guint len;
+
+  g_return_val_if_fail (str != NULL, NULL);
+
+  len = strlen (str);
+  str[len] = '\0';
+
+  return str;
+}
+
+static void
+git_init_email (void)
+{
+  char *user, *email;
+
+  /* Have an email for author and committer */
+  if (!git (&email, NULL, "config", "--get", "user.email", NULL) ||
+      email == NULL)
+    email = g_strdup ("flatpak-builder-commit@flatpak.org");
+  else
+    email = trim_linefeed (email);
+  g_setenv ("GIT_AUTHOR_EMAIL", email, FALSE);
+  g_setenv ("GIT_COMMITTER_EMAIL", email, FALSE);
+  g_free (email);
+
+  /* Have a "real name" for author and committer */
+  if (!git (&user, NULL, "config", "--get", "user.name", NULL) ||
+      user == NULL)
+    user = g_strdup ("Flatpak git committer");
+  else
+    user = trim_linefeed (user);
+  g_setenv ("GIT_AUTHOR_NAME", user, FALSE);
+  g_setenv ("GIT_COMMITTER_NAME", user, FALSE);
+  g_free (user);
+}
+
 int
 main (int    argc,
       char **argv)
@@ -312,9 +373,9 @@ main (int    argc,
   g_autoptr(BuilderManifest) manifest = NULL;
   g_autoptr(GOptionContext) context = NULL;
   const char *app_dir_path = NULL, *manifest_rel_path;
-  g_autofree gchar *json = NULL;
-  g_autofree gchar *json_sha256 = NULL;
-  g_autofree gchar *old_json_sha256 = NULL;
+  g_autofree gchar *manifest_contents = NULL;
+  g_autofree gchar *manifest_sha256 = NULL;
+  g_autofree gchar *old_manifest_sha256 = NULL;
   g_autoptr(BuilderContext) build_context = NULL;
   g_autoptr(GFile) base_dir = NULL;
   g_autoptr(GFile) manifest_file = NULL;
@@ -332,6 +393,7 @@ main (int    argc,
   gboolean is_run = FALSE;
   gboolean is_show_deps = FALSE;
   gboolean app_dir_is_empty = FALSE;
+  gboolean prune_unused_stages = FALSE;
   g_autoptr(FlatpakContext) arg_context = NULL;
   g_autoptr(FlatpakTempDir) cleanup_manifest_dir = NULL;
   g_autofree char *manifest_basename = NULL;
@@ -355,6 +417,15 @@ main (int    argc,
   else
     g_unsetenv ("GIO_USE_VFS");
 
+
+  /* Work around libsoup/glib race condition, as per:
+     https://bugzilla.gnome.org/show_bug.cgi?id=796031 and
+     https://bugzilla.gnome.org/show_bug.cgi?id=674885#c87 */
+  g_type_ensure (G_TYPE_SOCKET_FAMILY);
+  g_type_ensure (G_TYPE_SOCKET_TYPE);
+  g_type_ensure (G_TYPE_SOCKET_PROTOCOL);
+  g_type_ensure (G_TYPE_SOCKET_ADDRESS);
+
   orig_argv = g_memdup (argv, sizeof (char *) * argc);
   orig_argc = argc;
 
@@ -420,14 +491,12 @@ main (int    argc,
   manifest_rel_path = argv[argnr++];
   manifest_basename = g_path_get_basename (manifest_rel_path);
 
-#ifdef FLATPAK_ENABLE_P2P
   if (opt_collection_id != NULL &&
       !ostree_validate_collection_id (opt_collection_id, &error))
     {
       g_printerr ("‘%s’ is not a valid collection ID: %s", opt_collection_id, error->message);
       return 1;
     }
-#endif  /* FLATPAK_ENABLE_P2P */
 
   if (app_dir_path)
     app_dir = g_file_new_for_path (app_dir_path);
@@ -438,6 +507,7 @@ main (int    argc,
 
   builder_context_set_use_rofiles (build_context, !opt_disable_rofiles);
   builder_context_set_run_tests (build_context, !opt_disable_tests);
+  builder_context_set_no_shallow_clone (build_context, opt_no_shallow_clone);
   builder_context_set_keep_build_dirs (build_context, opt_keep_build_dirs);
   builder_context_set_delete_build_dirs (build_context, opt_delete_build_dirs);
   builder_context_set_sandboxed (build_context, opt_sandboxed);
@@ -445,6 +515,8 @@ main (int    argc,
   builder_context_set_rebuild_on_sdk_change (build_context, opt_rebuild_on_sdk_change);
   builder_context_set_bundle_sources (build_context, opt_bundle_sources);
 
+  git_init_email ();
+
   if (opt_sources_dirs)
     {
       g_autoptr(GPtrArray) sources_dirs = NULL;
@@ -463,6 +535,12 @@ main (int    argc,
       sources_urls = g_ptr_array_new_with_free_func ((GDestroyNotify)soup_uri_free);
       for (i = 0; opt_sources_urls[i] != NULL; i++)
         {
+          if (!g_str_has_suffix (opt_sources_urls[i], "/"))
+            {
+              g_autofree gchar *tmp = opt_sources_urls[i];
+              opt_sources_urls[i] = g_strdup_printf ("%s/", tmp);
+            }
+
           SoupURI *uri = soup_uri_new (opt_sources_urls[i]);
           if (uri == NULL)
             {
@@ -483,8 +561,7 @@ main (int    argc,
       builder_context_set_stop_at (build_context, opt_stop_at);
     }
 
-  if (opt_ccache &&
-      !builder_context_enable_ccache (build_context, &error))
+  if (!builder_context_set_enable_ccache (build_context, opt_ccache, &error))
     {
       g_printerr ("Can't initialize ccache use: %s\n", error->message);
       return 1;
@@ -536,18 +613,18 @@ main (int    argc,
 
   builder_context_set_base_dir (build_context, base_dir);
 
-  if (!g_file_get_contents (flatpak_file_get_path_cached (manifest_file), &json, NULL, &error))
+  if (!g_file_get_contents (flatpak_file_get_path_cached (manifest_file), &manifest_contents, NULL, &error))
     {
       g_printerr ("Can't load '%s': %s\n", manifest_rel_path, error->message);
       return 1;
     }
 
-  json_sha256 = g_compute_checksum_for_string (G_CHECKSUM_SHA256, json, -1);
+  manifest_sha256 = g_compute_checksum_for_string (G_CHECKSUM_SHA256, manifest_contents, -1);
 
   if (opt_skip_if_unchanged)
     {
-      old_json_sha256 = builder_context_get_checksum_for (build_context, manifest_basename);
-      if (old_json_sha256 != NULL && strcmp (json_sha256, old_json_sha256) == 0)
+      old_manifest_sha256 = builder_context_get_checksum_for (build_context, manifest_basename);
+      if (old_manifest_sha256 != NULL && strcmp (manifest_sha256, old_manifest_sha256) == 0)
         {
           g_print ("No changes to manifest, skipping\n");
           return 42;
@@ -557,8 +634,8 @@ main (int    argc,
   /* Can't push this as user data to the demarshalling :/ */
   builder_manifest_set_demarshal_base_dir (builder_context_get_base_dir (build_context));
 
-  manifest = (BuilderManifest *) json_gobject_from_data (BUILDER_TYPE_MANIFEST,
-                                                         json, -1, &error);
+  manifest = (BuilderManifest *) builder_gobject_from_data (BUILDER_TYPE_MANIFEST, manifest_rel_path,
+                                                            manifest_contents, &error);
 
   builder_manifest_set_demarshal_base_dir (NULL);
 
@@ -667,28 +744,29 @@ main (int    argc,
     }
 
   /* Verify that cache and build dir is on same filesystem */
-  {
-    g_autofree char *state_path = g_file_get_path (builder_context_get_state_dir (build_context));
-    g_autoptr(GFile) app_parent = g_file_get_parent (builder_context_get_app_dir (build_context));
-    g_autofree char *app_parent_path = g_file_get_path (app_parent);
-    struct stat buf1, buf2;
+  if (!opt_download_only)
+    {
+      g_autofree char *state_path = g_file_get_path (builder_context_get_state_dir (build_context));
+      g_autoptr(GFile) app_parent = g_file_get_parent (builder_context_get_app_dir (build_context));
+      g_autofree char *app_parent_path = g_file_get_path (app_parent);
+      struct stat buf1, buf2;
 
-    if (stat (app_parent_path, &buf1) == 0 && stat (state_path, &buf2) == 0 &&
-        buf1.st_dev != buf2.st_dev)
-      {
-        g_printerr ("The state dir (%s) is not on the same filesystem as the target dir (%s)\n",
-                    state_path, app_parent_path);
-        return 1;
-      }
-  }
+      if (stat (app_parent_path, &buf1) == 0 && stat (state_path, &buf2) == 0 &&
+          buf1.st_dev != buf2.st_dev)
+        {
+          g_printerr ("The state dir (%s) is not on the same filesystem as the target dir (%s)\n",
+                      state_path, app_parent_path);
+          return 1;
+        }
+    }
 
-  if (!builder_context_set_checksum_for (build_context, manifest_basename, json_sha256, &error))
+  if (!builder_context_set_checksum_for (build_context, manifest_basename, manifest_sha256, &error))
     {
       g_printerr ("Failed to set checksum for ‘%s’: %s\n", manifest_basename, error->message);
       return 1;
     }
 
-  if (!builder_manifest_start (manifest, opt_allow_missing_runtimes, build_context, &error))
+  if (!builder_manifest_start (manifest, opt_download_only, opt_allow_missing_runtimes, build_context, &error))
     {
       g_printerr ("Failed to init: %s\n", error->message);
       return 1;
@@ -793,7 +871,7 @@ main (int    argc,
         }
 
       if (builder_context_get_bundle_sources (build_context) &&
-          !builder_manifest_bundle_sources (manifest, json, cache, build_context, &error))
+          !builder_manifest_bundle_sources (manifest, manifest_contents, cache, build_context, &error))
         {
           g_printerr ("Error: %s\n", error->message);
           return 1;
@@ -813,7 +891,17 @@ main (int    argc,
       g_autoptr(GFile) cache = flatpak_build_file (builder_context_get_state_dir (build_context), "screenshots-cache", NULL);
       g_autoptr(GFile) screenshots = flatpak_build_file (app_dir, "screenshots", NULL);
       g_autoptr(GFile) screenshots_sub = flatpak_build_file (screenshots, screenshot_subdir, NULL);
+      g_autofree char *fs_app_dir = g_strdup_printf ("--filesystem=%s", flatpak_file_get_path_cached (app_dir));
+      g_autofree char *fs_cache = g_strdup_printf ("--filesystem=%s", flatpak_file_get_path_cached (cache));
       const char *argv[] = {
+        "flatpak",
+        "build",
+        "--die-with-parent",
+        "--nofilesystem=host",
+        fs_app_dir,
+        fs_cache,
+        "--share=network",
+        flatpak_file_get_path_cached (app_dir),
         "appstream-util",
         "mirror-screenshots",
         flatpak_file_get_path_cached (xml),
@@ -834,6 +922,16 @@ main (int    argc,
 
       if (g_file_query_exists (xml, NULL))
         {
+          if (!flatpak_mkdir_p (cache, NULL, &error))
+            {
+              g_printerr ("Error creating screenshot cache dir: %s\n", error->message);
+              return 1;
+            }
+          if (!flatpak_break_hardlink (xml, &error))
+            {
+              g_printerr ("Error mirroring screenshots: %s\n", error->message);
+              return 1;
+            }
           if (!builder_maybe_host_spawnv (NULL,
                                           NULL,
                                           0,
@@ -1054,7 +1152,10 @@ main (int    argc,
         }
     }
 
-  if (!builder_gc (cache, &error))
+  if (!opt_finish_only && !opt_export_only)
+    prune_unused_stages = TRUE;
+
+  if (!builder_gc (cache, prune_unused_stages, &error))
     {
       g_warning ("Failed to GC build cache: %s", error->message);
       g_clear_error (&error);
