@@ -341,6 +341,8 @@ builder_module_set_property (GObject      *object,
 
     case PROP_CMAKE:
       self->cmake = g_value_get_boolean (value);
+      if (self->cmake)
+        g_printerr ("The cmake module property is deprecated, use buildsystem cmake or cmake-ninja instead.\n");
       break;
 
     case PROP_BUILDSYSTEM:
@@ -719,10 +721,10 @@ builder_module_serialize_property (JsonSerializable *serializable,
     }
   else
     {
-      return json_serializable_default_serialize_property (serializable,
-                                                           property_name,
-                                                           value,
-                                                           pspec);
+      return builder_serializable_serialize_property (serializable,
+                                                      property_name,
+                                                      value,
+                                                      pspec);
     }
 }
 
@@ -824,14 +826,14 @@ builder_module_deserialize_property (JsonSerializable *serializable,
                   g_autoptr(GFile) module_file =
                     g_file_resolve_relative_path (saved_demarshal_base_dir, module_relpath);
                   const char *module_path = flatpak_file_get_path_cached (module_file);
-                  g_autofree char *json = NULL;
+                  g_autofree char *module_contents = NULL;
 
-                  if (g_file_get_contents (module_path, &json, NULL, NULL))
+                  if (g_file_get_contents (module_path, &module_contents, NULL, NULL))
                     {
                       g_autoptr(GFile) module_file_dir = g_file_get_parent (module_file);
                       builder_manifest_set_demarshal_base_dir (module_file_dir);
-                      module = json_gobject_from_data (BUILDER_TYPE_MODULE,
-                                                       json, -1, NULL);
+                      module = builder_gobject_from_data (BUILDER_TYPE_MODULE,
+                                                          module_relpath, module_contents, NULL);
                       builder_manifest_set_demarshal_base_dir (saved_demarshal_base_dir);
                       if (module)
                         {
@@ -910,10 +912,10 @@ builder_module_deserialize_property (JsonSerializable *serializable,
     }
   else
     {
-      return json_serializable_default_deserialize_property (serializable,
-                                                             property_name,
-                                                             value,
-                                                             pspec, property_node);
+      return builder_serializable_deserialize_property (serializable,
+                                                        property_name,
+                                                        value,
+                                                        pspec, property_node);
     }
 }
 
@@ -922,7 +924,10 @@ serializable_iface_init (JsonSerializableIface *serializable_iface)
 {
   serializable_iface->serialize_property = builder_module_serialize_property;
   serializable_iface->deserialize_property = builder_module_deserialize_property;
-  serializable_iface->find_property = builder_serializable_find_property_with_error;
+  serializable_iface->find_property = builder_serializable_find_property;
+  serializable_iface->list_properties = builder_serializable_list_properties;
+  serializable_iface->set_property = builder_serializable_set_property;
+  serializable_iface->get_property = builder_serializable_get_property;
 }
 
 const char *
@@ -1162,6 +1167,7 @@ setup_build_args (GFile          *app_dir,
   else
     builddir = "/run/build/";
 
+  g_ptr_array_add (args, g_strdup_printf ("--env=FLATPAK_BUILDER_BUILDDIR=%s%s", builddir, module_name));
   g_ptr_array_add (args, g_strdup ("--nofilesystem=host"));
 
   /* We mount the canonical location, because bind-mounts of symlinks don't really work */
@@ -1378,6 +1384,7 @@ builder_module_build_helper (BuilderModule  *self,
   GFile *app_dir = builder_context_get_app_dir (context);
   g_autofree char *make_j = NULL;
   g_autofree char *make_l = NULL;
+  g_autofree char *n_jobs = NULL;
   const char *make_cmd = NULL;
   const char *test_arg = NULL;
 
@@ -1425,6 +1432,9 @@ builder_module_build_helper (BuilderModule  *self,
   env = builder_options_get_env (self->build_options, context);
   config_opts = builder_options_get_config_opts (self->build_options, context, self->config_opts);
 
+  n_jobs = g_strdup_printf ("%d", self->no_parallel_make ? 1 : builder_context_get_jobs (context));
+  env = g_environ_setenv (env, "FLATPAK_BUILDER_N_JOBS", n_jobs, FALSE);
+
   if (!self->buildsystem)
     {
       if (self->cmake)
@@ -1557,6 +1567,8 @@ builder_module_build_helper (BuilderModule  *self,
       g_auto(GStrv) configure_args = NULL;
       g_autoptr(GPtrArray) configure_args_arr = g_ptr_array_new ();
       g_autofree char *configure_content = NULL;
+      const char *prefix = NULL;
+      const char *libdir = NULL;
 
       if (!g_file_load_contents (configure_file, NULL, &configure_content, NULL, NULL, error))
         {
@@ -1629,22 +1641,27 @@ builder_module_build_helper (BuilderModule  *self,
       else if (cmake_ninja)
         cmake_generator = "Ninja";
 
+      prefix = builder_options_get_prefix (self->build_options, context);
+      libdir = builder_options_get_libdir (self->build_options, context);
+
       if (cmake || cmake_ninja)
         {
-          g_ptr_array_add (configure_args_arr, g_strdup_printf ("-DCMAKE_INSTALL_PREFIX:PATH='%s'",
-                                                                builder_options_get_prefix (self->build_options, context)));
+          g_ptr_array_add (configure_args_arr, g_strdup_printf ("-DCMAKE_INSTALL_PREFIX:PATH='%s'", prefix));
+          if (libdir)
+            g_ptr_array_add (configure_args_arr, g_strdup_printf ("-DCMAKE_INSTALL_LIBDIR:PATH='%s'", libdir));
           g_ptr_array_add (configure_args_arr, g_strdup ("-G"));
           g_ptr_array_add (configure_args_arr, g_strdup_printf ("%s", cmake_generator));
         }
       else if (qmake)
         {
-          g_ptr_array_add (configure_args_arr, g_strdup_printf ("PREFIX='%s'",
-                                                                builder_options_get_prefix (self->build_options, context)));
+          g_ptr_array_add (configure_args_arr, g_strdup_printf ("PREFIX='%s'", prefix));
+          /* TODO: What parameter for qmake? */
         }
       else /* autotools and meson */
         {
-          g_ptr_array_add (configure_args_arr, g_strdup_printf ("--prefix=%s",
-                                                                builder_options_get_prefix (self->build_options, context)));
+          g_ptr_array_add (configure_args_arr, g_strdup_printf ("--prefix=%s", prefix));
+          if (libdir)
+            g_ptr_array_add (configure_args_arr, g_strdup_printf ("--libdir=%s", libdir));
         }
 
       g_ptr_array_add (configure_args_arr, configure_final_arg);
@@ -1694,6 +1711,11 @@ builder_module_build_helper (BuilderModule  *self,
       make_j = g_strdup_printf ("-j%d", builder_context_get_jobs (context));
       make_l = g_strdup_printf ("-l%d", 2 * builder_context_get_jobs (context));
     }
+  else if (meson || cmake_ninja)
+    {
+      /* ninja defaults to a parallel make, disable it if requested */
+      make_j = g_strdup ("-j1");
+    }
 
   if (run_shell)
     {
