@@ -40,13 +40,21 @@ struct BuilderOptions
   gboolean    no_debuginfo;
   gboolean    no_debuginfo_compression;
   char       *cflags;
+  gboolean    cflags_override;
   char       *cppflags;
+  gboolean    cppflags_override;
   char       *cxxflags;
+  gboolean    cxxflags_override;
   char       *ldflags;
+  gboolean    ldflags_override;
   char       *append_path;
+  char       *prepend_path;
   char       *append_ld_library_path;
+  char       *prepend_ld_library_path;
   char       *append_pkg_config_path;
+  char       *prepend_pkg_config_path;
   char       *prefix;
+  char       *libdir;
   char      **env;
   char      **build_args;
   char      **test_args;
@@ -69,10 +77,15 @@ G_DEFINE_TYPE_WITH_CODE (BuilderOptions, builder_options, G_TYPE_OBJECT,
 enum {
   PROP_0,
   PROP_CFLAGS,
+  PROP_CFLAGS_OVERRIDE,
   PROP_CPPFLAGS,
+  PROP_CPPFLAGS_OVERRIDE,
   PROP_CXXFLAGS,
+  PROP_CXXFLAGS_OVERRIDE,
   PROP_LDFLAGS,
+  PROP_LDFLAGS_OVERRIDE,
   PROP_PREFIX,
+  PROP_LIBDIR,
   PROP_ENV,
   PROP_STRIP,
   PROP_NO_DEBUGINFO,
@@ -84,8 +97,11 @@ enum {
   PROP_MAKE_ARGS,
   PROP_MAKE_INSTALL_ARGS,
   PROP_APPEND_PATH,
+  PROP_PREPEND_PATH,
   PROP_APPEND_LD_LIBRARY_PATH,
+  PROP_PREPEND_LD_LIBRARY_PATH,
   PROP_APPEND_PKG_CONFIG_PATH,
+  PROP_PREPEND_PKG_CONFIG_PATH,
   LAST_PROP
 };
 
@@ -100,9 +116,13 @@ builder_options_finalize (GObject *object)
   g_free (self->cppflags);
   g_free (self->ldflags);
   g_free (self->append_path);
+  g_free (self->prepend_path);
   g_free (self->append_ld_library_path);
+  g_free (self->prepend_ld_library_path);
   g_free (self->append_pkg_config_path);
+  g_free (self->prepend_pkg_config_path);
   g_free (self->prefix);
+  g_free (self->libdir);
   g_strfreev (self->env);
   g_strfreev (self->build_args);
   g_strfreev (self->test_args);
@@ -128,34 +148,66 @@ builder_options_get_property (GObject    *object,
       g_value_set_string (value, self->cflags);
       break;
 
+    case PROP_CFLAGS_OVERRIDE:
+      g_value_set_boolean (value, self->cflags_override);
+      break;
+
     case PROP_CPPFLAGS:
       g_value_set_string (value, self->cppflags);
       break;
 
+    case PROP_CPPFLAGS_OVERRIDE:
+      g_value_set_boolean (value, self->cppflags_override);
+      break;
+
     case PROP_CXXFLAGS:
       g_value_set_string (value, self->cxxflags);
       break;
 
+    case PROP_CXXFLAGS_OVERRIDE:
+      g_value_set_boolean (value, self->cxxflags_override);
+      break;
+
     case PROP_LDFLAGS:
       g_value_set_string (value, self->ldflags);
       break;
 
+    case PROP_LDFLAGS_OVERRIDE:
+      g_value_set_boolean (value, self->ldflags_override);
+      break;
+
     case PROP_APPEND_PATH:
       g_value_set_string (value, self->append_path);
       break;
 
+    case PROP_PREPEND_PATH:
+      g_value_set_string (value, self->prepend_path);
+      break;
+
     case PROP_APPEND_LD_LIBRARY_PATH:
       g_value_set_string (value, self->append_ld_library_path);
       break;
 
+    case PROP_PREPEND_LD_LIBRARY_PATH:
+      g_value_set_string (value, self->prepend_ld_library_path);
+      break;
+
     case PROP_APPEND_PKG_CONFIG_PATH:
       g_value_set_string (value, self->append_pkg_config_path);
       break;
 
+    case PROP_PREPEND_PKG_CONFIG_PATH:
+      g_value_set_string (value, self->prepend_pkg_config_path);
+      break;
+
     case PROP_PREFIX:
       g_value_set_string (value, self->prefix);
       break;
 
+    case PROP_LIBDIR:
+      g_value_set_string (value, self->libdir);
+      break;
+
     case PROP_ENV:
       g_value_set_boxed (value, self->env);
       break;
@@ -217,41 +269,77 @@ builder_options_set_property (GObject      *object,
       self->cflags = g_value_dup_string (value);
       break;
 
+    case PROP_CFLAGS_OVERRIDE:
+      self->cflags_override = g_value_get_boolean (value);
+      break;
+
     case PROP_CXXFLAGS:
       g_clear_pointer (&self->cxxflags, g_free);
       self->cxxflags = g_value_dup_string (value);
       break;
 
+    case PROP_CXXFLAGS_OVERRIDE:
+      self->cxxflags_override = g_value_get_boolean (value);
+      break;
+
     case PROP_CPPFLAGS:
       g_clear_pointer (&self->cppflags, g_free);
       self->cppflags = g_value_dup_string (value);
       break;
 
+    case PROP_CPPFLAGS_OVERRIDE:
+      self->cppflags_override = g_value_get_boolean (value);
+      break;
+
     case PROP_LDFLAGS:
       g_clear_pointer (&self->ldflags, g_free);
       self->ldflags = g_value_dup_string (value);
       break;
 
+    case PROP_LDFLAGS_OVERRIDE:
+      self->ldflags_override = g_value_get_boolean (value);
+      break;
+
     case PROP_APPEND_PATH:
       g_clear_pointer (&self->append_path, g_free);
       self->append_path = g_value_dup_string (value);
       break;
 
+    case PROP_PREPEND_PATH:
+      g_clear_pointer (&self->prepend_path, g_free);
+      self->prepend_path = g_value_dup_string (value);
+      break;
+
     case PROP_APPEND_LD_LIBRARY_PATH:
       g_clear_pointer (&self->append_ld_library_path, g_free);
       self->append_ld_library_path = g_value_dup_string (value);
       break;
 
+    case PROP_PREPEND_LD_LIBRARY_PATH:
+      g_clear_pointer (&self->prepend_ld_library_path, g_free);
+      self->prepend_ld_library_path = g_value_dup_string (value);
+      break;
+
     case PROP_APPEND_PKG_CONFIG_PATH:
       g_clear_pointer (&self->append_pkg_config_path, g_free);
       self->append_pkg_config_path = g_value_dup_string (value);
       break;
 
+    case PROP_PREPEND_PKG_CONFIG_PATH:
+      g_clear_pointer (&self->prepend_pkg_config_path, g_free);
+      self->prepend_pkg_config_path = g_value_dup_string (value);
+      break;
+
     case PROP_PREFIX:
       g_clear_pointer (&self->prefix, g_free);
       self->prefix = g_value_dup_string (value);
       break;
 
+    case PROP_LIBDIR:
+      g_clear_pointer (&self->libdir, g_free);
+      self->libdir = g_value_dup_string (value);
+      break;
+
     case PROP_ENV:
       tmp = self->env;
       self->env = g_strdupv (g_value_get_boxed (value));
@@ -327,6 +415,13 @@ builder_options_class_init (BuilderOptionsClass *klass)
                                                         "",
                                                         NULL,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_CFLAGS_OVERRIDE,
+                                   g_param_spec_boolean ("cflags-override",
+                                                         "",
+                                                         "",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
                                    PROP_CXXFLAGS,
                                    g_param_spec_string ("cxxflags",
@@ -334,6 +429,13 @@ builder_options_class_init (BuilderOptionsClass *klass)
                                                         "",
                                                         NULL,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_CXXFLAGS_OVERRIDE,
+                                   g_param_spec_boolean ("cxxflags-override",
+                                                         "",
+                                                         "",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
                                    PROP_CPPFLAGS,
                                    g_param_spec_string ("cppflags",
@@ -341,6 +443,13 @@ builder_options_class_init (BuilderOptionsClass *klass)
                                                         "",
                                                         NULL,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_CPPFLAGS_OVERRIDE,
+                                   g_param_spec_boolean ("cppflags-override",
+                                                         "",
+                                                         "",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
                                    PROP_LDFLAGS,
                                    g_param_spec_string ("ldflags",
@@ -348,6 +457,13 @@ builder_options_class_init (BuilderOptionsClass *klass)
                                                         "",
                                                         NULL,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_LDFLAGS_OVERRIDE,
+                                   g_param_spec_boolean ("ldflags-override",
+                                                         "",
+                                                         "",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
                                    PROP_APPEND_PATH,
                                    g_param_spec_string ("append-path",
@@ -355,6 +471,13 @@ builder_options_class_init (BuilderOptionsClass *klass)
                                                         "",
                                                         NULL,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_PREPEND_PATH,
+                                   g_param_spec_string ("prepend-path",
+                                                        "",
+                                                        "",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
                                    PROP_APPEND_LD_LIBRARY_PATH,
                                    g_param_spec_string ("append-ld-library-path",
@@ -362,6 +485,13 @@ builder_options_class_init (BuilderOptionsClass *klass)
                                                         "",
                                                         NULL,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_PREPEND_LD_LIBRARY_PATH,
+                                   g_param_spec_string ("prepend-ld-library-path",
+                                                        "",
+                                                        "",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
                                    PROP_APPEND_PKG_CONFIG_PATH,
                                    g_param_spec_string ("append-pkg-config-path",
@@ -369,6 +499,13 @@ builder_options_class_init (BuilderOptionsClass *klass)
                                                         "",
                                                         NULL,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_PREPEND_PKG_CONFIG_PATH,
+                                   g_param_spec_string ("prepend-pkg-config-path",
+                                                        "",
+                                                        "",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
                                    PROP_PREFIX,
                                    g_param_spec_string ("prefix",
@@ -376,6 +513,13 @@ builder_options_class_init (BuilderOptionsClass *klass)
                                                         "",
                                                         NULL,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (object_class,
+                                   PROP_LIBDIR,
+                                   g_param_spec_string ("libdir",
+                                                        "",
+                                                        "",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
   g_object_class_install_property (object_class,
                                    PROP_ENV,
                                    g_param_spec_boxed ("env",
@@ -446,6 +590,7 @@ builder_options_class_init (BuilderOptionsClass *klass)
                                                          "",
                                                          FALSE,
                                                          G_PARAM_READWRITE));
+
 }
 
 static void
@@ -626,7 +771,8 @@ serializable_iface_init (JsonSerializableIface *serializable_iface)
 {
   serializable_iface->serialize_property = builder_options_serialize_property;
   serializable_iface->deserialize_property = builder_options_deserialize_property;
-  serializable_iface->find_property = builder_serializable_find_property_with_error;
+  serializable_iface->find_property = builder_serializable_find_property;
+  serializable_iface->list_properties = builder_serializable_list_properties;
 }
 
 static GList *
@@ -661,12 +807,21 @@ get_all_options (BuilderOptions *self, BuilderContext *context)
 }
 
 static const char *
-builder_options_get_flags (BuilderOptions *self, BuilderContext *context, size_t field_offset)
+builder_options_get_flags (BuilderOptions *self,
+                           BuilderContext *context,
+                           size_t          field_offset,
+                           size_t          override_field_offset,
+                           const char     *sdk_flags)
 {
   g_autoptr(GList) options = get_all_options (self, context);
   GList *l;
   GString *flags = NULL;
 
+  if (sdk_flags && sdk_flags[0])
+    {
+      flags = g_string_new (sdk_flags);
+    }
+
   /* Last command flag wins, so reverse order */
   options = g_list_reverse (options);
 
@@ -674,6 +829,10 @@ builder_options_get_flags (BuilderOptions *self, BuilderContext *context, size_t
     {
       BuilderOptions *o = l->data;
       const char *flag = G_STRUCT_MEMBER (const char *, o, field_offset);
+      gboolean override = G_STRUCT_MEMBER (gboolean, o, override_field_offset);
+
+      if (override && flags)
+        g_string_truncate (flags, 0);
 
       if (flag)
         {
@@ -693,32 +852,49 @@ builder_options_get_flags (BuilderOptions *self, BuilderContext *context, size_t
   return NULL;
 }
 
+static const char *
+get_sdk_flags (BuilderOptions *self, BuilderContext *context, const char *(*method)(BuilderSdkConfig *self))
+{
+  BuilderSdkConfig *sdk_config = builder_context_get_sdk_config (context);
+  if (sdk_config)
+    return (*method) (sdk_config);
+  return NULL;
+}
+
 const char *
 builder_options_get_cflags (BuilderOptions *self, BuilderContext *context)
 {
-  return builder_options_get_flags (self, context, G_STRUCT_OFFSET (BuilderOptions, cflags));
+  return builder_options_get_flags (self, context, G_STRUCT_OFFSET (BuilderOptions, cflags),
+                                    G_STRUCT_OFFSET (BuilderOptions, cflags_override),
+                                    get_sdk_flags (self, context, builder_sdk_config_get_cflags));
 }
 
 const char *
 builder_options_get_cxxflags (BuilderOptions *self, BuilderContext *context)
 {
-  return builder_options_get_flags (self, context, G_STRUCT_OFFSET (BuilderOptions, cxxflags));
+  return builder_options_get_flags (self, context, G_STRUCT_OFFSET (BuilderOptions, cxxflags),
+                                    G_STRUCT_OFFSET (BuilderOptions, cxxflags_override),
+                                    get_sdk_flags (self, context, builder_sdk_config_get_cxxflags));
 }
 
 const char *
 builder_options_get_cppflags (BuilderOptions *self, BuilderContext *context)
 {
-  return builder_options_get_flags (self, context, G_STRUCT_OFFSET (BuilderOptions, cppflags));
+  return builder_options_get_flags (self, context, G_STRUCT_OFFSET (BuilderOptions, cppflags),
+                                    G_STRUCT_OFFSET (BuilderOptions, cppflags_override),
+                                    get_sdk_flags (self, context, builder_sdk_config_get_cppflags));
 }
 
 const char *
 builder_options_get_ldflags (BuilderOptions *self, BuilderContext *context)
 {
-  return builder_options_get_flags (self, context, G_STRUCT_OFFSET (BuilderOptions, ldflags));
+  return builder_options_get_flags (self, context, G_STRUCT_OFFSET (BuilderOptions, ldflags),
+                                    G_STRUCT_OFFSET (BuilderOptions, ldflags_override),
+                                    get_sdk_flags (self, context, builder_sdk_config_get_ldflags));
 }
 
 static char *
-builder_options_get_appended_path (BuilderOptions *self, BuilderContext *context, const char *initial_value, size_t field_offset)
+builder_options_get_appended_path (BuilderOptions *self, BuilderContext *context, const char *initial_value, size_t append_field_offset, size_t prepend_field_offset)
 {
   g_autoptr(GList) options = get_all_options (self, context);
   GList *l;
@@ -730,7 +906,8 @@ builder_options_get_appended_path (BuilderOptions *self, BuilderContext *context
   for (l = options; l != NULL; l = l->next)
     {
       BuilderOptions *o = l->data;
-      const char *append = G_STRUCT_MEMBER (const char *, o, field_offset);
+      const char *append = G_STRUCT_MEMBER (const char *, o, append_field_offset);
+      const char *prepend = G_STRUCT_MEMBER (const char *, o, prepend_field_offset);
 
       if (append)
         {
@@ -742,6 +919,17 @@ builder_options_get_appended_path (BuilderOptions *self, BuilderContext *context
 
           g_string_append (path_list, append);
         }
+
+      if (prepend)
+        {
+          if (path_list == NULL)
+            path_list = g_string_new ("");
+
+          if (path_list->len > 0)
+            g_string_prepend_c (path_list, ':');
+
+          g_string_prepend (path_list, prepend);
+        }
     }
 
   if (path_list)
@@ -761,7 +949,8 @@ builder_options_update_ld_path (BuilderOptions *self, BuilderContext *context, c
     old = "/app/lib";
 
   path = builder_options_get_appended_path (self, context, old,
-                                            G_STRUCT_OFFSET (BuilderOptions, append_ld_library_path));
+                                            G_STRUCT_OFFSET (BuilderOptions, append_ld_library_path),
+                                            G_STRUCT_OFFSET (BuilderOptions, prepend_ld_library_path));
   if (path)
     envp = g_environ_setenv (envp, "LD_LIBRARY_PATH", path, TRUE);
 
@@ -779,7 +968,8 @@ builder_options_update_pkg_config_path (BuilderOptions *self, BuilderContext *co
     old = "/app/lib/pkgconfig:/app/share/pkgconfig:/usr/lib/pkgconfig:/usr/share/pkgconfig";
 
   path = builder_options_get_appended_path (self, context, old,
-                                            G_STRUCT_OFFSET (BuilderOptions, append_pkg_config_path));
+                                            G_STRUCT_OFFSET (BuilderOptions, append_pkg_config_path),
+                                            G_STRUCT_OFFSET (BuilderOptions, prepend_pkg_config_path));
   if (path)
     envp = g_environ_setenv (envp, "PKG_CONFIG_PATH", path, TRUE);
 
@@ -792,7 +982,8 @@ builder_options_update_path (BuilderOptions *self, BuilderContext *context, char
   g_autofree char *path = NULL;
   path = builder_options_get_appended_path (self, context,
                                             g_environ_getenv (envp, "PATH"),
-                                            G_STRUCT_OFFSET (BuilderOptions, append_path));
+                                            G_STRUCT_OFFSET (BuilderOptions, append_path),
+                                            G_STRUCT_OFFSET (BuilderOptions, prepend_path));
   if (path)
     envp = g_environ_setenv (envp, "PATH", path, TRUE);
   return envp;
@@ -817,6 +1008,25 @@ builder_options_get_prefix (BuilderOptions *self, BuilderContext *context)
   return "/app";
 }
 
+const char *
+builder_options_get_libdir (BuilderOptions *self, BuilderContext *context)
+{
+  g_autoptr(GList) options = get_all_options (self, context);
+  GList *l;
+
+  for (l = options; l != NULL; l = l->next)
+    {
+      BuilderOptions *o = l->data;
+      if (o->libdir)
+        return o->libdir;
+    }
+
+  if (builder_context_get_build_runtime (context))
+    return get_sdk_flags (self, context, builder_sdk_config_get_libdir);
+
+  return NULL;
+}
+
 gboolean
 builder_options_get_strip (BuilderOptions *self, BuilderContext *context)
 {
@@ -951,10 +1161,17 @@ builder_options_get_build_args (BuilderOptions *self,
         }
     }
 
-  if (array->len > 0 && builder_context_get_sandboxed (context))
+  if (builder_context_get_sandboxed (context))
     {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, "Can't specify build-args in sandboxed build");
-      return NULL;
+      if (array->len > 0)
+        {
+          g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, "Can't specify build-args in sandboxed build");
+          return NULL;
+        }
+      /* If, for whatever reason, the app has network access in the
+         metadata, explicitly neuter that if we're building
+         sandboxed */
+      g_ptr_array_add (array, g_strdup ("--unshare=network"));
     }
 
   g_ptr_array_add (array, NULL);
@@ -1071,10 +1288,15 @@ builder_options_checksum (BuilderOptions *self,
 
   builder_cache_checksum_str (cache, BUILDER_OPTION_CHECKSUM_VERSION);
   builder_cache_checksum_str (cache, self->cflags);
+  builder_cache_checksum_compat_boolean (cache, self->cflags_override);
   builder_cache_checksum_str (cache, self->cxxflags);
+  builder_cache_checksum_compat_boolean (cache, self->cxxflags_override);
   builder_cache_checksum_str (cache, self->cppflags);
+  builder_cache_checksum_compat_boolean (cache, self->cppflags_override);
   builder_cache_checksum_str (cache, self->ldflags);
+  builder_cache_checksum_compat_boolean (cache, self->ldflags_override);
   builder_cache_checksum_str (cache, self->prefix);
+  builder_cache_checksum_compat_str (cache, self->libdir);
   builder_cache_checksum_strv (cache, self->env);
   builder_cache_checksum_strv (cache, self->build_args);
   builder_cache_checksum_compat_strv (cache, self->test_args);
@@ -1086,8 +1308,11 @@ builder_options_checksum (BuilderOptions *self,
   builder_cache_checksum_boolean (cache, self->no_debuginfo_compression);
 
   builder_cache_checksum_compat_str (cache, self->append_path);
+  builder_cache_checksum_compat_str (cache, self->prepend_path);
   builder_cache_checksum_compat_str (cache, self->append_ld_library_path);
+  builder_cache_checksum_compat_str (cache, self->prepend_ld_library_path);
   builder_cache_checksum_compat_str (cache, self->append_pkg_config_path);
+  builder_cache_checksum_compat_str (cache, self->prepend_pkg_config_path);
 
   arch_options = g_hash_table_lookup (self->arch, builder_context_get_arch (context));
   if (arch_options)
